Package: proxy
Type: Package
Title: Distance and Similarity Measures
Version: 0.4-29
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"),
                    email = "David.Meyer@R-project.org",
		    comment = c(ORCID = "0000-0002-5196-3048")),	    
             person(given = "Christian", family = "Buchta", role = "aut"))
Description: Provides an extensible framework for the efficient calculation of auto- and cross-proximities, along with implementations of the most popular ones. 
Depends: R (>= 3.4.0)
Imports: stats, utils
Suggests: cba
Collate: registry.R database.R dist.R similarities.R dissimilarities.R
        util.R seal.R
License: GPL-2 | GPL-3
NeedsCompilation: yes
Packaged: 2025-12-29 16:26:15 UTC; hornik
Author: David Meyer [aut, cre] (ORCID: <https://orcid.org/0000-0002-5196-3048>),
  Christian Buchta [aut]
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2025-12-29 16:42:24 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:01:35 UTC; unix
Archs: proxy.so.dSYM

Package: proxy
Type: Package
Title: Distance and Similarity Measures
Version: 0.4-27
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"), email = "David.Meyer@R-project.org"),
             person(given = "Christian", family = "Buchta", role = "aut"))
Description: Provides an extensible framework for the efficient calculation of auto- and cross-proximities, along with implementations of the most popular ones. 
Depends: R (>= 3.4.0)
Imports: stats, utils
Suggests: cba
Collate: registry.R database.R dist.R similarities.R dissimilarities.R
        util.R seal.R
License: GPL-2
NeedsCompilation: yes
Packaged: 2022-06-08 21:36:02 UTC; meyer
Author: David Meyer [aut, cre],
  Christian Buchta [aut]
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2022-06-09 06:15:32 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 02:15:09 UTC; unix

Package: proxy
Type: Package
Title: Distance and Similarity Measures
Version: 0.4-29
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"),
                    email = "David.Meyer@R-project.org",
		    comment = c(ORCID = "0000-0002-5196-3048")),	    
             person(given = "Christian", family = "Buchta", role = "aut"))
Description: Provides an extensible framework for the efficient calculation of auto- and cross-proximities, along with implementations of the most popular ones. 
Depends: R (>= 3.4.0)
Imports: stats, utils
Suggests: cba
Collate: registry.R database.R dist.R similarities.R dissimilarities.R
        util.R seal.R
License: GPL-2 | GPL-3
NeedsCompilation: yes
Packaged: 2025-12-29 16:26:15 UTC; hornik
Author: David Meyer [aut, cre] (ORCID: <https://orcid.org/0000-0002-5196-3048>),
  Christian Buchta [aut]
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2025-12-29 16:42:24 UTC

import("stats")
importFrom("utils", "formatUL")

export("dist", "simil",
       "as.dist", "as.simil", "as.matrix",
       "pr_simil2dist", "pr_dist2simil",
       "row.dist", "col.dist",
       "rowSums.dist", "colSums.dist", "rowMeans.dist", "colMeans.dist",
       "pr_DB")

S3method("dim", "dist")
S3method("dimnames", "dist")
S3method("dimnames<-", "dist")
S3method("names", "dist")
S3method("names<-", "dist")

S3method("subset", "dist")
S3method("[[", "dist")
S3method("as.matrix", "dist")
S3method("as.matrix", "simil")
S3method("print","crossdist")
S3method("print","pairdist")
S3method("print","simil")

S3method("summary","pr_DB")
S3method("print","summary.pr_DB")

## generic registry stuff
S3method("[[", "proxy_registry")
S3method("length", "proxy_registry")
S3method("print","proxy_registry")
S3method("print","registry_field")
S3method("print","registry_entry")
S3method("summary","proxy_registry")
S3method("as.data.frame","proxy_registry")

useDynLib("proxy", .registration = TRUE)

### ceeboo 2007

## todo: special values NaN, NA, and Inf

library("proxy")

set.seed(20070630)

##

x <- matrix(runif(20),5,4)
inherits(x, "matrix")
rownames(x) <- LETTERS[1:5]
y <- x
y[2,] <- x[1,] <- 0

x
y

## user interfaces

r <- .Call("R_minkowski_dist", x, NULL, FALSE, 1, PACKAGE = "proxy")
all.equal(c(r), c(stats::dist(x, method = "minkowski", p = 1)))
r
.Call("R_minkowski_dist", x, x, FALSE, 1, PACKAGE = "proxy")
.Call("R_minkowski_dist", x, y, FALSE, 1, PACKAGE = "proxy")
.Call("R_minkowski_dist", x, y, TRUE,  1, PACKAGE = "proxy")

dfun <- paste("R",c("euclidean", "maximum", "manhattan", "canberra", "binary", "matching", "fuzzy", "mutual"),"dist", sep = "_")

for (f in dfun) {
    cat("\nTesting ",f,"\n\n",sep="")
    r <- try(do.call(".Call", list(f, x, NULL, FALSE, PACKAGE = "proxy")))
    if ( inherits(r, "try-error"))
	next
    s <- try(stats::dist(x, method = gsub("R_|_dist", "", f)))
    if (!inherits(s, "try-error"))
        print(all.equal(c(r), c(s)))
    print(r)
    print(do.call(".Call", list(f, x, x, FALSE, PACKAGE = "proxy")))
    print(do.call(".Call", list(f, x, y, FALSE, PACKAGE = "proxy")))
    print(do.call(".Call", list(f, x, y, TRUE,  PACKAGE = "proxy")))
}

## no longer optimized

.Call("R_ejaccard", x, NULL, FALSE, PACKAGE = "proxy")
.Call("R_ejaccard", x, x, FALSE, PACKAGE = "proxy")
.Call("R_ejaccard", x, y, FALSE, PACKAGE = "proxy")
.Call("R_ejaccard", x, y, TRUE, PACKAGE = "proxy")

.Call("R_cosine", x, NULL, FALSE, PACKAGE = "proxy")
.Call("R_cosine", x, x, FALSE, PACKAGE = "proxy")
.Call("R_cosine", x, y, FALSE, PACKAGE = "proxy")
.Call("R_cosine", x, y, TRUE, PACKAGE = "proxy")

x <- matrix(x > 0.5, 5,4)
y <- matrix(y > 0.5, 5,4)

.Call("R_bjaccard", x, NULL, FALSE, PACKAGE = "proxy")
.Call("R_bjaccard", x, x, FALSE, PACKAGE = "proxy")
.Call("R_bjaccard", x, y, FALSE, PACKAGE = "proxy")
.Call("R_bjaccard", x, y, TRUE, PACKAGE = "proxy")

###

## tests on apply C wrappers

library(proxy)

set.seed(20070630)

## matrix

f <- function(x, y) sum(x*y) / sqrt(sum(x*x)) / sqrt(sum(y*y))

x <- matrix(runif(20), 5, 4)
x
y <- matrix(runif(20), 5, 4)
y

.External(proxy:::R_apply_dist_matrix, x, NULL, FALSE, f)
.External(proxy:::R_apply_dist_matrix, x, x, FALSE, f)
.External(proxy:::R_apply_dist_matrix, x, y, FALSE, f)
.External(proxy:::R_apply_dist_matrix, x, y, TRUE, f)

# coerce

z <- y * 100
storage.mode(z) <- "integer"
.External(proxy:::R_apply_dist_matrix, x, z, FALSE, f)
.External(proxy:::R_apply_dist_matrix, z, x, FALSE, f)
.External(proxy:::R_apply_dist_matrix, z, z, FALSE, f)
.External(proxy:::R_apply_dist_matrix, z, NULL, FALSE, f)

## list

x <- unlist(apply(x, 1, list), recursive = FALSE)
x
y <- unlist(apply(y, 1, list), recursive = FALSE)

.External(proxy:::R_apply_dist_list, x, NULL, FALSE, f)
.External(proxy:::R_apply_dist_list, x, x, FALSE, f)
.External(proxy:::R_apply_dist_list, x, y, FALSE, f)
.External(proxy:::R_apply_dist_list, x, y, TRUE, f)

## logical matrix

f <- function(a, b, c, d, n)
    a / sqrt(a+b) / sqrt(a+c)

x <- t(sapply(x, ">", 0.5))
x
y <- t(sapply(y, ">", 0.5))

.External(proxy:::R_apply_dist_binary_matrix, x, NULL, FALSE, f)
.External(proxy:::R_apply_dist_binary_matrix, x, x, FALSE, f)
.External(proxy:::R_apply_dist_binary_matrix, x, y, FALSE, f)
.External(proxy:::R_apply_dist_binary_matrix, x, y, TRUE, f)

## data.frame

f <- function(x, y) sum(x*y) / sqrt(sum(x*x)) / sqrt(sum(y*y))

x <- data.frame(unlist(apply(x, 2, list), recursive = FALSE))
names(x) <- letters[1:4]
x
y <- data.frame(unlist(apply(y, 2, list), recursive = FALSE))
names(y) <- letters[1:4]

.External(proxy:::R_apply_dist_data_frame, x, NULL, FALSE, f)
.External(proxy:::R_apply_dist_data_frame, x, x, FALSE, f)
.External(proxy:::R_apply_dist_data_frame, x, y, FALSE, f)
.External(proxy:::R_apply_dist_data_frame, x, y, TRUE, f)

#

f <- function(x, y) {
    if (rownames(x) == 1 && rownames(y) == 1) {
        print(x)
        str(x)
        print(y)
    }
    sum(x == y) / length(x)
}

x <- data.frame(1:5, LETTERS[1:5], stringsAsFactors = FALSE)
x

y <- data.frame(1:6, LETTERS[c(1,1:5)],
                row.names = letters[1:6],
                stringsAsFactors = FALSE)
y

all.equal(x, y)
identical(attributes(x[[1]]), attributes(y[[1]]))
identical(attributes(x[[2]]), attributes(y[[2]]))

.External(proxy:::R_apply_dist_data_frame, x, NULL, FALSE, f)
.External(proxy:::R_apply_dist_data_frame, x, x, FALSE, f)
.External(proxy:::R_apply_dist_data_frame, x, y, FALSE, f)
.External(proxy:::R_apply_dist_data_frame, x, y[-6,], TRUE, f)


# test parameters

f <- function(x, y, p = 1)
    sum(x == y) / length(x) * p

.External(proxy:::R_apply_dist_data_frame, x, y[-6,], TRUE, f)
.External(proxy:::R_apply_dist_data_frame, x, y[-6,], TRUE, f, p = 2)

###

#################################
## Test for dist calls
##################################

library(proxy)

set.seed(20140107)

## get all measures
proxies = pr_DB$get_entry_names()

## remove special cases
proxies = setdiff(proxies, c("Mahalanobis", "Minkowski", "Stiles", "Levenshtein", "fJaccard"))

## create test data
x = matrix(1:100, 10)

## test function: checks if dist(x) == dist(x,x) for all measures,
## and if diag(dist(x, x)) == diag(x, x, pairwise = TRUE)
prtest <- function(...) {
    CD <- dist(x, x, ...)
    all(as.matrix(dist(x, ...)) == CD) &&
      all(diag(CD) == dist(x, x, pairwise = TRUE, ...))
}

## loop over all measures (except special cases)
for (i in proxies)
    {cat(i); prtest(i); cat(": OK.\n")}

## Minkowski
for (j in c(0.5, 1, 2, 3, Inf))
    {cat("Minkowski: p =", j); prtest("Minkowski", p = j); cat(": OK.\n")}

## Mahalanobis (need non-singular matrix)
x = as.matrix(iris[1:50,-5])
prtest("Mahalanobis")

## fJaccard (needs values in unit interval)
x = as.matrix(1:100/100, 10)
prtest("fJaccard")

## produce binary matrix
x = matrix(rbinom(100,1,0.7), 10)

## Stiles (gives a lot of warnings due to log)
tmp = dist(x, "Stiles")
tmp = dist(x, x, "Stiles")

## try again (almost) all measures, this time with binary data to check
## conversions
for (i in proxies)
    {cat(i); prtest(i); cat(": OK.\n")}
## Minkowski
for (j in c(0.5, 1, 2, 3, Inf))
    {cat("Minkowski: p =", j); prtest("Minkowski", p = j); cat(": OK.\n")}

## Levenshtein distance
s <- c("A", "quick", "brown", "fox", "jumps", "over", "the", "lazy", "dog")
all(as.matrix(dist(s, "Levenshtein")) == dist(s, s, "Levenshtein"))

## Test auto-conversion
x = iris[,-5]
prtest()

######################
### proximity database
######################

### check functions
.function_or_character <-
function(x)
{
    if (!is.character(x) && !is.function(x))
        stop("Need function or function name.")
}

.abcd_and_binary <-
function(x)
{
    if (x$abcd && (x$type != "binary"))
        stop(paste(dQuote("abcd"), "mode only available for binary measures."))
}

### create registry
pr_DB <- registry(registry_class = "pr_DB",
                  entry_class = "proxy_registry_entry",
                  validity_FUN = ".abcd_and_binary")

## create fields
pr_DB$set_field("FUN", is_mandatory = TRUE, validity_FUN = ".function_or_character")
pr_DB$set_field("distance", type = "logical", default = TRUE)
pr_DB$set_field("PREFUN", validity_FUN = ".function_or_character")
pr_DB$set_field("POSTFUN", validity_FUN = ".function_or_character")
pr_DB$set_field("convert", validity_FUN = ".function_or_character")
pr_DB$set_field("type", type = c("binary", "nominal", "ordinal", "metric", "other"),
                default = "other")
pr_DB$set_field("loop", type = "logical", default = TRUE)
pr_DB$set_field("C_FUN", type = "logical", default = FALSE)
pr_DB$set_field("PACKAGE", type = "character", default = "proxy")
pr_DB$set_field("abcd", type = "logical", default = FALSE)
pr_DB$set_field("formula", type = "character")
pr_DB$set_field("reference", type = "character")
pr_DB$set_field("description", type = "character")

### summary and print methods
summary.pr_DB <-
function(object, verbosity = c("short", "long"), ...)
{
    if (length(object) < 1)
        return(object)
    verbosity <- match.arg(verbosity)

    object <-
        switch(verbosity,
               short = list(names = object$get_field_entries("names", unlist = FALSE),
                            distance = object$get_field_entries("distance")),
               long = list(names = object$get_field_entries("names", unlist = FALSE),
                           distance = object$get_field_entries("distance"),
                           type = object$get_field_entries("type"),
                           formula = object$get_field_entries("formula"))
               )

    structure(object, class = "summary.pr_DB")
}

print.summary.pr_DB <- function(x, ...)
{
    distance <- c("Similarity", "Distance")[x[[2]] + 1]
    if (length(x) > 2)
        x[[3]][is.na(x[[3]])] <- "other"
    for (i in unique(distance)) {
        ind <- which(distance == i)
        if (length(ind) > 0) {
            writeLines(paste("*", i, "measures:"))
            if (length(x) > 2) {
                for (k in ind)
                    writeLines(paste("     ", paste(x[[1]][[k]], collapse = "/"),
                                     " (", x[[3]][k], ") = ", x[[4]][k], sep = ""))
            } else {
                tmp <- sort(sapply(x[[1]][ind], function(i) i[1]))
                writeLines(strwrap(paste(tmp, collapse = ", ")))
            }
        }
        writeLines("")
    }
}

dist <-
function(x, y = NULL, method = NULL, ...,
         diag = FALSE, upper = FALSE, pairwise = FALSE,
         by_rows = TRUE, convert_similarities = TRUE,
         auto_convert_data_frames = TRUE)
{

### PARAMETER HANDLING
    ## convenience hack to allow dist(x, "method")
    if ((is.function(y) || is.character(y)) && is.null(method)) {
        method <- y
        y <- NULL
    }

    ## transform data frame into matrix iff all columns are atomic and either numeric (integer or double) or logical or complex
    is.n_l_c <- function(x)
        all(sapply(x, is.numeric)) ||
        all(sapply(x, is.logical)) ||
        all(sapply(x, is.complex))
    if (is.data.frame(x) && auto_convert_data_frames && is.n_l_c(x))
        x <- as.matrix(x)
    if (is.data.frame(y) && !is.null(y) && auto_convert_data_frames && is.n_l_c(y))
        y <- as.matrix(y)

    ## vector handling
    if (is.vector(x) && is.atomic(x))
        x <- as.matrix(x)
    if (!is.null(y) && is.vector(y) && is.atomic(y))
        y <- as.matrix(y)

    ## method lookup
    reg_entry <- NULL
    if (is.null(method))
        method <- if (is.data.frame(x))
            "Gower"
        else if (is.logical(x))
            "Jaccard"
        else
            "Euclidean"
    if (!is.function(method))
        reg_entry <- if (inherits(method, "proxy_registry_entry"))
            method
        else
            pr_DB$get_entry(method)

    ## some checks
    if (!is.data.frame(x) && !is.matrix(x) && !is.list(x))
        stop("Can only handle data frames, vectors, matrices, and lists!")
    if ( is.data.frame(x) && !by_rows)
        stop("Cannot transpose mixed data frames")
    if (!is.null(y)) {
        if (is.data.frame(x) && !is.data.frame(y)
            || is.matrix(x) && !is.matrix(y)
            || is.list(x) && !is.list(y))
            stop("x and y must be of same type.")
        if (is.matrix(x) && is.matrix(y) || is.data.frame(x) && is.data.frame(y))
            if (by_rows && (ncol(x) != ncol(y)))
                stop("x and y must be conform in columns.")
            else if (!by_rows && (nrow(x) != nrow(y)))
                stop("x and y must be conform in rows.")
    }

### PREPROCESS
    params <- list(...)
    if (!is.null(reg_entry)) {
        if(is.function(reg_entry$PREFUN) || is.character(reg_entry$PREFUN)) {
            tmp <- do.call(reg_entry$PREFUN,
                           c(list(x, y, pairwise, params, reg_entry)))
            if (!is.null(tmp)) {
                x <- tmp$x
                y <- tmp$y
                pairwise <- tmp$pairwise
                params <- tmp$p
                reg_entry <- tmp$reg_entry
            }
        }
        method <- reg_entry$FUN
    }

    ## helper function for calling the C-level loops
    .proxy_external <- function(CFUN, x, y)
        do.call(".External",
                c(list(CFUN, x, y, pairwise,
                       if (!is.function(method)) get(method) else method),
                  params
                  )
                )

    result <-
### PASS-THROUGH-cases
        if (!is.null(reg_entry) && !reg_entry$loop) {
            if (!by_rows && !is.list(x)) {
                x <- t(x)
                if (!is.null(y))
                    y <- t(y)
            }
            if (reg_entry$C_FUN) {
		do.call(".Call", c(list(method), list(x), list(y), pairwise, params, list(PACKAGE = reg_entry$PACKAGE)))
	    } else    ## user functions need not implement pairwise
                do.call(method, c(list(x), list(y), params), envir = asNamespace(reg_entry$PACKAGE))
        } else if (is.null(y)) {
### LOOP WORKHORSE for auto-proximities
            ## transpose data for column-wise loop
            if (!by_rows && !is.list(x))
                x <- t(x)
            if (is.list(x) && !is.null(reg_entry) && reg_entry$abcd)
                x <- do.call("rbind", x)
            if (is.matrix(x) && !is.null(reg_entry) && reg_entry$abcd)
                ## binary matrix
                .proxy_external(R_apply_dist_binary_matrix, x != 0, NULL)
            else if (is.matrix(x))
                ## real, integer matrix
                .proxy_external(R_apply_dist_matrix, x, NULL)
            else if (is.list(x) && !(is.data.frame(x) && by_rows))
                ## list
                .proxy_external(R_apply_dist_list, x, NULL)
            else ## data frame (by rows)
                .proxy_external(R_apply_dist_data_frame, x, NULL)

        } else {
### LOOP WORKHORSE for cross-proximities
            ## transpose data for column-wise loop
            if (!by_rows && !is.list(x)) {
                x <- t(x)
                y <- t(y)
            }
            if (is.list(x) && !is.null(reg_entry) && reg_entry$abcd)
            {
                x <- do.call("rbind", x)
                y <- do.call("rbind", x)
            }
            if (is.matrix(x) && !is.null(reg_entry) && reg_entry$abcd)
                ## binary matrices
                .proxy_external(R_apply_dist_binary_matrix, x != 0, y != 0)
            else if (is.matrix(x))
                ## real, integer matrices
                .proxy_external(R_apply_dist_matrix, x, y)
            else if (is.list(x) && !(is.data.frame(x) && by_rows))
                ## lists
                .proxy_external(R_apply_dist_list, x, y)
            else ## data frames (by rows)
                .proxy_external(R_apply_dist_data_frame, x, y)

        }

### set col/rownames for cross-proximity-objects (if needed)
    if (is.matrix(result) && is.null(dimnames(result)))
        if (is.list(x) && !is.data.frame(x)) {
            rownames(result) <- names(x)
            colnames(result) <- names(y)
        } else if (by_rows) {
            rownames(result) <- rownames(x)
            colnames(result) <- rownames(y)
        } else {
            rownames(result) <- colnames(x)
            colnames(result) <- colnames(y)
        }

### POSTPROCESS
    if (!is.null(reg_entry)) {
        if (is.function(reg_entry$POSTFUN) || is.character(reg_entry$POSTFUN))
            result <- do.call(reg_entry$POSTFUN, c(list(result, params)))
        if (!reg_entry$distance &&
            !(is.logical(convert_similarities) && !convert_similarities)) {
            result <- if (is.function(convert_similarities) ||
                          is.character(convert_similarities))
                do.call(convert_similarities, list(result))
            else if (is.null(reg_entry$convert))
                pr_simil2dist(result)
            else
                do.call(reg_entry$convert, list(result))
        }
        method <- reg_entry$names[1]
    }

### RETURN DIST-OBJECT
    result <-
        if (is.matrix(result))
            structure(result, class = "crossdist")
        else
        if (inherits(result, "dist"))
            structure(result, Diag = diag, Upper = upper)
        else
            structure(result, class = "pairdist")
    structure(result,
              method = if (is.character(method))
                            method
                       else
                          if (missing(method))
                               deparse(substitute(y))
                          else deparse(substitute(method)),
              call = match.call())
}

simil <-
function(x, y = NULL, method = NULL, ...,
         diag = FALSE, upper = FALSE, pairwise = FALSE,
         by_rows = TRUE, convert_distances = TRUE,
         auto_convert_data_frames = TRUE)
{
    ## convenience to allow dists(x, "method")
    if ((is.function(y) || is.character(y)) && is.null(method)) {
        method <- y
        y <- NULL
    }
    if (is.null(method))
        method <- if (is.data.frame(x))
            "Gower"
        else if (is.logical(x))
            "Jaccard"
        else
            "correlation"

    ret <- dist(x, y, method, ..., diag = diag, upper = upper, pairwise = pairwise,
                by_rows = by_rows, convert_similarities = FALSE,
                auto_convert_data_frames = auto_convert_data_frames)

    ## possibly convert to similarity
    reg_entry <- pr_DB$get_entry(attr(ret, "method"), stop_if_missing = FALSE)
    if (!is.null(reg_entry)) {
        if (reg_entry$distance &&
            !(is.logical(convert_distances) && !convert_distances)) {
            ret <- if (is.function(convert_distances) ||
                       is.character(convert_distances))
                do.call(convert_distances, list(ret))
            else if (is.null(reg_entry$convert))
                pr_simil2dist(ret)
            else
                do.call(reg_entry$convert, list(ret))
        }
    }

    class(ret) <- unique(c(if (inherits(ret, "crossdist")) "crosssimil" else "simil",
                           class(ret)))
    ret
}

# note that a simil object must always also be a dist
# object for method dispatch

as.simil <-
function(x, FUN = NULL)
{
    if (inherits(x, c("simil", "crosssimil")))
        x
    else if (inherits(x, c("dist", "crossdist"))) {
        class(x) <- if (inherits(x, "dist"))
            c("simil", class(x))
        else
            c("crosssimil", setdiff(class(x), "crossdist"))
        if (!is.null(FUN))
            FUN(x)
        else {
            reg_entry <- NULL
            if (!is.null(attr(x, "method")))
                reg_entry <- pr_DB$get_entry(attr(x, "method"),
                                             stop_if_missing = FALSE)
            if (!is.null(reg_entry) && !is.null(reg_entry$convert))
                do.call(reg_entry$convert, list(x))
            else
                pr_dist2simil(x)
        }
    } else
        structure(stats::as.dist(x), class = c("simil", "dist"))
}

as.dist <-
function(x, FUN = NULL)
{
    if (inherits(x, c("simil", "crosssimil"))) {
        class(x) <- if (inherits(x, "simil"))
            setdiff(class(x), "simil")
        else
            c("crossdist", setdiff(class(x), "crosssimil"))
        if (!is.null(FUN))
            FUN(x)
        else {
            reg_entry <- NULL
            if (!is.null(attr(x, "method")))
                reg_entry <- pr_DB$get_entry(attr(x, "method"),
                                             stop_if_missing = FALSE)
            if (!is.null(reg_entry) && !is.null(reg_entry$convert))
                do.call(reg_entry$convert, list(x))
            else
                pr_simil2dist(x)
        }
    } else if (inherits(x, c("dist", "crossdist")))
        x
    else
        stats::as.dist(x)
}

## as we do not know if the object is the result of some
## user-defined transformation the values of
## s(x,x) are not defined.

## we need to copy stats::as.matrix.dist() since the use of ::: is deprecated:

as.matrix <-
function(x, ...)
    base::as.matrix(x, ...)

as_matrix_dist <-
function (x, ...)
{
    size <- attr(x, "Size")
    df <- matrix(0, size, size)
    df[row(df) > col(df)] <- x
    df <- df + t(df)
    labels <- attr(x, "Labels")
    dimnames(df) <- if (is.null(labels))
        list(seq_len(size), seq_len(size))
    else list(labels, labels)
    df
}


as.matrix.simil <-
function(x, diag = NA, ...) {
    x <- as_matrix_dist(x)
    diag(x) <- diag
    x
}

## however, it seems reasonable to assume that d(x,x)=0,
## which is also the default in stats.

as.matrix.dist <-
function(x, diag = 0, ...) {
    x <- as_matrix_dist(x)
    diag(x) <- diag
    x
}

print.crossdist <-
print.crosssimil <-
function (x, digits = getOption("digits"),
          justify = "none", right = TRUE, ...)
{
    if (length(x) > 0) {
        m <- as.matrix(x)
        cf <- format(m, digits = digits, justify = justify)
        print(cf, quote = FALSE, right = right, ...)
    } else {
        cat(data.class(x), "(0)\n", sep = "")
    }
    invisible(x)
}

print.pairdist <-
function(x, ...)
{
    print(as.vector(x), ...)
    invisible(x)
}

print.simil <-
function (x, diag = NULL, upper = NULL, digits = getOption("digits"),
    justify = "none", right = TRUE, ...)
{
    if (length(x)) {
        if (is.null(diag))
            diag <- if (is.null(a <- attr(x, "Diag")))
                FALSE
            else a
        if (is.null(upper))
            upper <- if (is.null(a <- attr(x, "Upper")))
                FALSE
            else a
        m <- as.matrix(x)
        if (diag) diag(m) <- NA
        cf <- format(m, digits = digits, justify = justify)
        if (!upper)
            cf[row(cf) < col(cf)] <- ""
        if (!diag)
            cf[row(cf) == col(cf)] <- ""
        print(if (diag || upper)
            cf
        else cf[-1, -attr(x, "Size"), drop = FALSE], quote = FALSE,
            right = right, ...)
    }
    else {
        cat(data.class(x), "(0)\n", sep = "")
    }
    invisible(x)
}

pr_simil2dist <-
function(x)
    1 - abs(x)

pr_dist2simil <-
function(x)
    1 / (1 + x)

###

## test C interfaces

library(proxy)

set.seed(20070630)

x <- as.dist(matrix(runif(25),5,5))
x
attributes(x)

z <- .Call(proxy:::R_subset_dist, x, 3)
z

unclass(z)

.Call(proxy:::R_subset_dist, x, c(1,3,5))

attr(x, "Labels") <- LETTERS[1:5]

z <- .Call(proxy:::R_subset_dist, x, c("A","C","E"))
z
attributes(z)

attr(x, "Labels") <- NULL

.Call(proxy:::R_rowSums_dist, x, FALSE)
.Call(proxy:::R_rowSums_dist, z, FALSE)

.Call(proxy:::R_row_dist, x, FALSE)       # row()
.Call(proxy:::R_row_dist, x, TRUE)        # col()

## test R interfaces

dim(x)
dimnames(x) <- letters[1:5]
dimnames(x)
names(x)    <- LETTERS[1:5]
names(x)

row.dist(x)
col.dist(x)

subset(x, c(1,3,5))
x[[c(1,3,5)]]
x[c(1,3,5)]                         # as usual

x[[-1]]                             # drop subscripts

x[[1]]                              # empty

###

## wrappers for class dist
##
## note that all type checking and coercing
## is now done in C, as well as handling of
## attributes.
##
## fixme: create generic functions?
##
## ceeboo 2007

dim.dist <-
function(x)
    rep.int(attr(x, "Size"), 2)         # works with nrow and ncol

dimnames.dist <-
names.dist <-
function(x)
    attr(x, "Labels")

"dimnames<-.dist" <-
"names<-.dist" <-
function(x, value)
{
    if (is.null(value))
        attr(x, "Labels") <- NULL
    else {
        if (length(value) != attr(x, "Size"))
            stop("dimension of 'x' and length of 'value' do not conform")
        attr(x, "Labels") <- as.character(value)
    }
    x
}

row.dist <-
function(x)
    .Call(R_row_dist, x, FALSE)

col.dist <-
function(x)
    .Call(R_row_dist, x, TRUE)

##

subset.dist <-
"[[.dist" <-
function(x, subset, ...)
{
    if (missing(subset))
        return(x)
    .Call(R_subset_dist, x, unique(subset))
}

##

rowSums.dist <-
colSums.dist <-
function(x, na.rm = FALSE)
    .Call(R_rowSums_dist, x, na.rm)

##

rowMeans.dist <-
colMeans.dist <-
function(x, na.rm = FALSE, diag = TRUE)
{
    if (!is.logical(diag))
        stop("'diag' not of type logical")
    s <- rowSums.dist(x, na.rm)
    if (na.rm) {
        x[!(is.na(x) | is.nan(x))] <- 1
        s / (rowSums.dist(x, na.rm) + (diag == TRUE))
    } else
        s / (length(s) - (diag == FALSE))
}

###

pr_Euclidean <- function(x, y) sqrt(crossprod(x - y))
pr_Euclidean_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_Euclidean"
    }
    list(x = if (!is.list(x)) 0 + x else x,
         y = if (!is.null(y)) if (!is.list(y)) 0 + y else y,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_euclidean_dist",
                names = c("Euclidean","L2"),
                PREFUN = "pr_Euclidean_prefun",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "sqrt(sum_i (x_i - y_i)^2))",
                reference = "Cox, T.F., and Cox, M.A.A. (2001. Multidimensional Scaling. Chapmann and Hall.",
                description = "The Euclidean Distance (C implementation with compensation for excluded components)")

pr_Mahalanobis <- function(x, y, cov) sqrt(mahalanobis(x, y, cov))
pr_Mahalanobis_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (length(p) < 1) p <- list(cov(x, y))
    list(x = x, y = y, pairwise = pairwise, p = p, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "pr_Mahalanobis",
                names = "Mahalanobis",
                PREFUN = "pr_Mahalanobis_prefun",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt((x - y) Sigma^(-1) (x - y))",
                reference = "Mahalanobis P.C. (1936), On the generalised distance in statistics, Proceedings of the National Institute of Science of India 12, pp. 49-55",
                description = "The Mahalanobis Distance. The Variance-Covariance-Matrix is estimated from the input data if unspecified.")

pr_Bhjattacharyya <- function(x, y) sqrt(crossprod(sqrt(x) - sqrt(y)))
pr_DB$set_entry(FUN = "pr_Bhjattacharyya",
                names = "Bhjattacharyya",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt(sum_i (sqrt(x_i) - sqrt(y_i))^2))",
                reference = "Bhattacharyya A. (1943). On a measure of divergence between two statistical populations defined by probability distributions, Bull. Calcutta Math. Soc., vol. 35, pp. 99--109",
                description = "The Bhjattacharyya Distance")

pr_Manhattan <- function(x, y) sum(abs(x - y))
pr_Manhattan_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_Manhattan"
    }
    list(x = if (!is.list(x)) 0 + x else x,
         y = if (!is.null(y)) if (!is.list(y)) 0 + y else y,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_manhattan_dist",
                names = c("Manhattan", "City-Block", "L1", "taxi"),
                PREFUN = "pr_Manhattan_prefun",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "sum_i |x_i - y_i|",
                reference = "Cox, T.F., and Cox, M.A.A. (2001. Multidimensional Scaling. Chapmann and Hall.",
                description = "The Manhattan/City-Block/Taxi/L1-Distance (C implementation with compensation for excluded components)")

pr_supremum <- function(x, y) max(abs(x - y))
pr_supremum_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_supremum"
    }
    list(x = if (!is.list(x)) 0 + x else x,
         y = if (!is.null(y)) if (!is.list(y)) 0 + y else y,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_maximum_dist",
                names = c("supremum", "max", "maximum", "Tschebyscheff", "Chebyshev"),
                distance = TRUE,
                PREFUN = "pr_supremum_prefun",
                convert = "pr_dist2simil",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "max_i |x_i - y_i|",
                reference = "Cox, T.F., and Cox, M.A.A. (2001. Multidimensional Scaling. Chapmann and Hall.",
                description = "The Maximum/Supremum/Chebyshev Distance (C implementation)")

pr_Minkowski <- function(x, y, p = 2) (sum(abs(x - y) ^ p)) ^ (1/p)
pr_Minkowski_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (length(p$p) < 1)
        stop("Argument 'p' mandatory!")
    p <- p$p[[1L]]
    if (p <= 0)
        stop("'p' must not be smaller than 0.")
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_Minkowski"
    }
    list(x = if (!is.list(x)) 0 + x else x,
         y = if (!is.null(y)) if (!is.list(y)) 0 + y else y,
         pairwise = pairwise,
         p = p, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_minkowski_dist",
                names = c("Minkowski","Lp"),
                PREFUN = "pr_Minkowski_prefun",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "(sum_i (x_i - y_i)^p)^(1/p)",
                reference = "Cox, T.F., and Cox, M.A.A. (2001. Multidimensional Scaling. Chapmann and Hall.",
                description = "The Minkowski Distance (C implementation with compensation for excluded components)")

pr_Canberra <- function(x, y) {tmp <- abs(x - y) / abs(x + y); sum(tmp[!is.nan(tmp)])}
pr_Canberra_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_Canberra"
    }
    list(x = if (!is.list(x)) 0 + x else x,
         y = if (!is.null(y)) if (!is.list(y)) 0 + y else y,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_canberra_dist",
                names = "Canberra",
                PREFUN = "pr_Canberra_prefun",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "sum_i |x_i - y_i| / |x_i + y_i|",
                reference = "Cox, T.F., and Cox, M.A.A. (2001. Multidimensional Scaling. Chapmann and Hall.",
                description = "The Canberra Distance (C implementation with compensation for excluded components)")

pr_WaveHedges <- function(x, y) sum(1 - min(x, y) / max(x, y))
pr_DB$set_entry(FUN = "pr_WaveHedges",
                names = c("Wave", "Hedges"),
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_i (1 - min(x_i, y_i) / max(x_i, y_i))",
                reference = "Cox, T.F., and Cox, M.A.A. (2001). Multidimensional Scaling. Chapmann and Hall.",
                description = "The Wave/Hedges Distance")

pr_Divergence <- function(x, y) {tmp <- (x - y)^2 / (x + y)^2; sum(tmp[!is.nan(tmp)])}
pr_DB$set_entry(FUN = "pr_Divergence",
                names = "divergence",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_i (x_i - y_i)^2 / (x_i + y_i)^2",
                reference = "Cox, T.F., and Cox, M.A.A. (2001). Multidimensional Scaling. Chapmann and Hall.",
                description = "The Divergence Distance")

pr_KullbackLeibler <- function(x,y) {
    p <- x / sum(x);
    q <- y / sum(y);
    sum(p * log(p / q))
}
pr_DB$set_entry(FUN = "pr_KullbackLeibler",
                names = c("Kullback", "Leibler"),
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_i [x_i * log((x_i / sum_j x_j) / (y_i / sum_j y_j)) / sum_j x_j)]",
                reference = "Kullback S., and Leibler, R.A. (1951). On information and sufficiency. The Annals of Mathematical Statistics, vol. 22, pp. 79--86",
                description = "The Kullback-Leibler-distance.")

pr_BrayCurtis <- function(x, y) sum(abs(x - y)) / sum(x + y)
pr_DB$set_entry(FUN = "pr_BrayCurtis",
                names = c("Bray","Curtis"),
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_i |x_i - y_i| / sum_i (x_i + y_i)",
                reference = "Bray J.R., Curtis J.T. (1957). An ordination of the upland forest of the southern Winsconsin. Ecological Monographies, 27, pp. 325--349",
                description = "The Bray/Curtis dissimilarity. Note that it is not a distance since it vioalates the triangle inequality.")

pr_Soergel <- function(x, y) sum(abs(x - y)) / sum(max(x, y))
pr_DB$set_entry(FUN = "pr_Soergel",
                names = "Soergel",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_i |x_i - y_i| / sum_i max{x_i, y_i}",
                reference = "Cox, T.F., and Cox, M.A.A. (2001). Multidimensional Scaling. Chapmann and Hall.",
                description = "The Soergel Distance")

pr_Levenshtein_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (system.file(package="cba") == "")
        stop("Need package 'cba'!")
    else
        loadNamespace("cba")
    if (pairwise)
        stop("Pairwise distances not implemented by sdist()!")
    list(x = x, y = y, pairwise = pairwise, p = p, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "sdists",
                names = "Levenshtein",
                PREFUN = "pr_Levenshtein_prefun",
                convert = "pr_dist2simil",
                distance = TRUE,
                loop = FALSE,
                abcd = FALSE,
                C_FUN = FALSE,
                PACKAGE = "cba",
                formula = "Number of insertions, edits, and deletions between to strings",
                reference = "Levenshtein V.I. (1966). Binary codes capable of correcting deletions, insertions, and reversals. Soviet Physics Doklady 10, pp. 707--710",
                description = "Wrapper for sdists() in the cba-package (C implementation).")


pr_Podani <- function(x, y) {
    a <- b <- c <- d <- 0
    n <- length(x)
    for (i in seq_len(n - 1))
        for(j in (i+1):n) {
            a <- a + (x[i] < x[j] && y[i] < y[j] || x[i] > x[j] && y[i] > y[j])
            b <- b + (x[i] < x[j] && y[i] > y[j] || x[i] > x[j] && y[i] < y[j])
            c <- c + (x[i] == x[j] && y[i] == y[j] &&
                      (x[i] == 0 && y[i] == 0 || x[i] > 0 && y[i] > 0))

            z <- sum(x[i] == 0, x[j] == 0, y[i] == 0, y[j] == 0)
            d <- d + ((x[i] == x[j] || y[i] == y[j]) && z > 0 && z < 4)
        }
    1 - 2 * (a - b + c - d) / (n * (n - 1))
}

pr_DB$set_entry(FUN = "pr_Podani",
                names = c("Podani","discordance"),
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "1 - 2 * (a - b + c - d) / (n * (n - 1))",
                reference = "Podani, J. (1997). A measure of discordance for partially ranked data when presence/absence is also meaningful. Coenoses 12: 127--130.",
                description = "The Podany measure of discordance is defined on ranks with ties. In the formula, for two given objects x and y, n is the number of variables, a is is the number of pairs of variables ordered identically, b the number of pairs reversely ordered, c the number of pairs tied in both x and y (corresponding to either joint presence or absence), and d the number of all pairs of variables tied at least for one of the objects compared such that one, two, or thee scores are zero.")

pr_chord <- function(x, y) sqrt(2 * (1 - crossprod(x, y) / sqrt(crossprod(x) * crossprod(y))))
pr_DB$set_entry(FUN = pr_chord,
                names = "Chord",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt(2 * (1 - xy / sqrt(xx * yy)))",
                reference = "Orloci, L. 1967. An agglomerative method for classification of plant communities. J. Ecol 55:193--206.",
                description = "The Chord distance.")

pr_geodesic <- function(x, y) acos(crossprod(x, y) / sqrt(crossprod(x) * crossprod(y)))
pr_DB$set_entry(FUN = pr_geodesic,
                names = "Geodesic",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "arccos(xy / sqrt(xx * yy))",
                reference = "Orloci, L. 1967. Data centering: a review and evaluation with reference to component analysis. Syst. Zool. 16:208--212.",
                description = "The geoedesic distance, i.e. the angle between x and y.")

pr_whittaker <- function(x, y) sum(abs(x / sum(x) - y / sum(y))) / 2
pr_DB$set_entry(FUN = pr_whittaker,
                names = "Whittaker",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_i |x_i / sum_i x - y_i / sum_i y| / 2",
                reference = "Whittaker, R.H. (1952) A study of summer foliage insect communities in the Great Smoky Mountains. Ecological Monographs 22, pp. 1--44.",
                description = "The Whittaker distance.")

pr_hellinger <- function(x, y) sqrt(crossprod(sqrt(x / sum(x)) - sqrt(y / sum(y))))
pr_DB$set_entry(FUN = pr_hellinger,
                names = "Hellinger",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt(sum_i (sqrt(x_i / sum_i x) - sqrt(y_i / sum_i y)) ^ 2)",
                reference = "Rao, C.R. (1995) Use of Hellinger distance in graphical displays. In E.-M. Tiit, T. Kollo, & H. Niemi (Ed.): Multivariate statistics and matrices in statistics. Leiden (Netherland): Brill Academic Publisher. pp. 143--161.",
                description = "The Hellinger distance.")


pr_fJaccard <- function(x, y) sum(pmin(x, y)) / sum(pmax(x, y))
pr_fJaccard_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (any(x < 0 | x > 1))
        stop("Valid range for fuzzy measure: 0 <= x <= 1")
    if (!is.null(y) && any(y < 0 | y > 1))
        stop("Valid range for fuzzy measure: 0 <= y <= 1")

    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_fJaccard"
    }
    list(x = 0 + x,
         y = if (!is.null(y)) 0 + y else NULL,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_fuzzy_dist",
                names = c("fJaccard", "fuzzy_Jaccard"),
                PREFUN = "pr_fJaccard_prefun",
                distance = TRUE,
                convert = "pr_dist2simil",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "sum_i (min{x_i, y_i} / max{x_i, y_i})",
                reference = "Miyamoto S. (1990). Fuzzy sets in information retrieval and cluster analysis, Kluwer Academic Publishers, Dordrecht.",
                description = "The fuzzy Jaccard dissimilarity (C implementation).")

### Binary measures
pr_Jaccard <- function(a, b, c, d, n) a / (n - d)
pr_Jaccard_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$abcd <- TRUE
        reg_entry$FUN <- "pr_Jaccard"
    } else {
        storage.mode(x) <- "logical"
        if (!is.null(y))
            storage.mode(y) <- "logical"
    }

    list(x = x, y = y, pairwise = pairwise, p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_bjaccard",
                names = c("Jaccard","binary","Reyssac","Roux"),
                distance = FALSE,
                PREFUN = "pr_Jaccard_prefun",
                convert = "pr_simil2dist",
                type = "binary",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "a / (a + b + c)",
                reference = "Jaccard, P. (1908). Nouvelles recherches sur la distribution florale. Bull. Soc. Vaud. Sci. Nat., 44, pp. 223--270.",
                description = "The Jaccard Similarity (C implementation) for binary data. It is the proportion of (TRUE, TRUE) pairs, but not considering (FALSE, FALSE) pairs. So it compares the intersection with the union of object sets.")

pr_Kulczynski1 <- function(a, b, c, d, n) a / (b + c)
pr_DB$set_entry(FUN = "pr_Kulczynski1",
                names = "Kulczynski1",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "a / (b + c)",
                reference = "Kurzcynski, T.W. (1970). Generalized distance and discrete variables. Biometrics, 26, pp. 525--534.",
                description = "Kulczynski Similarity for binary data. Relates the (TRUE, TRUE) pairs to discordant pairs.")

pr_Kulczynski2 <- function(a, b, c, d, n) (a / (a + b) + a / (a + c)) / 2
pr_DB$set_entry(FUN = "pr_Kulczynski2",
                names = "Kulczynski2",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "[a / (a + b) + a / (a + c)] / 2",
                reference = "Kurzcynski, T.W. (1970). Generalized distance and discrete variables. Biometrics, 26, pp. 525--534.",
                description = "Kulczynski Similarity for binary data. Relates the (TRUE, TRUE) pairs to the discordant pairs.")

pr_Mountford <- function(a, b, c, d, n) 2 * a / (a * (b + c) + 2 * b * c)
pr_DB$set_entry(FUN = "pr_Mountford",
                names = "Mountford",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "2a / (ab + ac + 2bc)",
                reference = "Mountford, M.D. (1962). An index of similarity and its application to classificatory probems. In P.W. Murphy (ed.), Progress in Soil Zoology, pp. 43--50. Butterworth, London.",
                description = "The Mountford Similarity for binary data.")

pr_fagerMcgowan <- function(a, b, c, d, n) a / sqrt((a + b) * (a + c)) - sqrt(a + c) / 2
pr_DB$set_entry(FUN = "pr_fagerMcgowan",
                names = c("Fager", "McGowan"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "a / sqrt((a + b)(a + c)) - sqrt(a + c) / 2",
                reference = "Fager, E. W. and McGowan, J. A. (1963). Zooplankton species groups in the North Pacific. Science, N. Y. 140: 453-460",
                description = "The Fager / McGowan distance.")


pr_RusselRao <- function(a, b, c, d, n) a / n
pr_DB$set_entry(FUN = "pr_RusselRao",
                names = c("Russel","Rao"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "a / n",
                reference = "Russell, P.F., and Rao T.R. (1940). On habitat and association of species of anopheline larvae in southeastern, Madras, J. Malaria Inst. India 3, pp. 153--178",
                description = "The Russel/Rao Similarity for binary data. It is just the proportion of (TRUE, TRUE) pairs.")

pr_SimpleMatching <- function(a, b, c, d, n) (a + d) / n
pr_DB$set_entry(FUN = "pr_SimpleMatching",
                names = c("simple matching", "Sokal/Michener"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "(a + d) / n",
                reference = "Sokal, R.R., and Michener, C.D. (1958). A statistical method for evaluating systematic relationships. Univ. Kansas Sci. Bull., 39, pp. 1409--1438.",
                description = "The Simple Matching Similarity or binary data. It is the proportion of concordant pairs.")

pr_Hamman <- function(a, b, c, d, n) (a + d - b - c) / n
pr_DB$set_entry(FUN = "pr_Hamman",
                names = "Hamman",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "([a + d] - [b + c]) / n",
                reference = "Hamann, U. (1961). Merkmalbestand und Verwandtschaftsbeziehungen der Farinosae. Ein Beitrag zum System der Monokotyledonen. Willdenowia, 2, pp. 639-768.",
                description = "The Hamman Matching Similarity for binary data. It is the proportion difference of the concordant and discordant pairs.")

pr_Faith <- function(a, b, c, d, n) (a + d / 2) / n
pr_DB$set_entry(FUN = "pr_Faith",
                names = "Faith",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "(a + d/2) / n",
                reference = "Belbin, L., Marshall, C. & Faith, D.P. (1983). Representing relationships by automatic assignment of colour. The Australian Computing Journal 15, 160-163.",
                description = "The Faith similarity")

pr_RogersTanimoto <- function(a, b, c, d, n) (a + d) / (a + 2 * (b + c) + d)
pr_DB$set_entry(FUN = "pr_RogersTanimoto",
                names = c("Tanimoto", "Rogers"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "(a + d) / (a + 2b + 2c + d)",
                reference = "Rogers, D.J, and Tanimoto, T.T. (1960). A computer program for classifying plants. Science, 132, pp. 1115--1118.",
                description = "The Rogers/Tanimoto Similarity for binary data. Similar to the simple matching coefficient, but putting double weight on the discordant pairs.")

pr_Dice <- function(a, b, c, d, n) 2 * a / (2 * a + b + c)
pr_DB$set_entry(FUN = "pr_Dice",
                names = c("Dice", "Czekanowski", "Sorensen"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "2a / (2a + b + c)",
                reference = "Dice, L.R. (1945). Measures of the amount of ecologic association between species. Ecolology, 26, pp. 297--302.",
                description = "The Dice Similarity")

pr_Phi <- function(a, b, c, d, n)
    (a * d - b * c) / (sqrt(a + b) * sqrt(c + d) * sqrt(a + c) * sqrt(b + d))
pr_DB$set_entry(FUN = "pr_Phi",
                names = "Phi",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "(ad - bc) / sqrt[(a + b)(c + d)(a + c)(b + d)]",
                reference = "Sokal, R.R, and Sneath, P.H.A. (1963). Principles of numerical taxonomy. W.H. Freeman and Company, San Francisco.",
                description = "The Phi Similarity (= Product-Moment-Correlation for binary variables)")

pr_Stiles <- function(a, b, c, d, n)
    log(n) + 2 * log(abs(a * d - b * c) - 0.5 * n) - log(a + b) - log(c + d) - log(a + c) - log(b + d)
pr_DB$set_entry(FUN = "pr_Stiles",
                names = "Stiles",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "log(n(|ad-bc| - 0.5n)^2 / [(a + b)(c + d)(a + c)(b + d)])",
                reference = "Stiles, H.E. (1961). The association factor in information retrieval. Communictions of the ACM, 8, 1, pp. 271--279.",
                description = "The Stiles Similarity (used for information retrieval). Identical to the logarithm of Krylov's distance.")

pr_Michael <- function(a, b, c, d, n)
    4 * (a * d - b * c) / ((a + d) * (a + d) + (b + c) * (b + c))
pr_DB$set_entry(FUN = "pr_Michael",
                names = "Michael",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "4(ad - bc) / [(a + d)^2 + (b + c)^2]",
                reference = "Cox, T.F., and Cox, M.A.A. (2001). Multidimensional Scaling. Chapmann and Hall.",
                description = "The Michael Similarity")

pr_MozleyMargalef <- function(a, b, c, d, n)
    a * n / ((a + b) * (a + c))
pr_DB$set_entry(FUN = "pr_MozleyMargalef",
                names = c("Mozley","Margalef"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "an / (a + b)(a + c)",
                reference = "Margalef, D.R. (1958). Information theory in ecology. Gen. Systems, 3, pp. 36--71.",
                description = "The Mozley/Margalef Similarity")

pr_Yule<- function(a, b, c, d, n) {
    ad <- a * d
    bc <- b * c
    (ad - bc) / (ad + bc)
}
pr_DB$set_entry(FUN = "pr_Yule",
                names = "Yule",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "(ad - bc) / (ad + bc)",
                reference = "Yule, G.U. (1912). On measuring associations between attributes. J. Roy. Stat. Soc., 75, pp. 579--642.",
                description = "Yule Similarity")

pr_Yule2<- function(a, b, c, d, n) {
    ad <- a * d
    bc <- b * c
    (sqrt(ad) - sqrt(bc)) / (sqrt(ad) + sqrt(bc))
}
pr_DB$set_entry(FUN = "pr_Yule2",
                names = "Yule2",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "(sqrt(ad) - sqrt(bc)) / (sqrt(ad) + sqrt(bc))",
                reference = "Yule, G.U. (1912). On measuring associations between attributes. J. Roy. Stat. Soc., 75, pp. 579--642.",
                description = "Yule Similarity")

pr_Ochiai <- function(a, b, c, d, n)
    a / sqrt((a + b) * (a + c))
pr_DB$set_entry(FUN = "pr_Ochiai",
                names = "Ochiai",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "a / sqrt[(a + b)(a + c)]",
                reference = "Sokal, R.R, and Sneath, P.H.A. (1963). Principles of numerical taxonomy. W.H. Freeman and Company, San Francisco.",
                description = "The Ochiai Similarity")

pr_Simpson <- function(a, b, c, d, n)
    a / min((a + b), (a + c))
pr_DB$set_entry(FUN = "pr_Simpson",
                names = "Simpson",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "a / min{(a + b), (a + c)}",
                reference = "Simpson, G.G. (1960). Notes on the measurement of faunal resemblance. American Journal of Science 258-A: 300-311.",
                description = "The Simpson Similarity (used in Zoology).")

pr_BraunBlanquet <- function(a, b, c, d, n)
    a / max((a + b), (a + c))
pr_DB$set_entry(FUN = "pr_BraunBlanquet",
                names = c("Braun-Blanquet"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "binary",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = TRUE,
                formula = "a / max{(a + b), (a + c)}",
                reference = "Braun-Blanquet, J. (1964): Pflanzensoziologie. Springer Verlag, Wien and New York.",
                description = "The Braun-Blanquet Similarity (used in Biology).")


pr_cos <- function(x, y) abs(crossprod(x, y) / sqrt(crossprod(x) * crossprod(y)))
pr_cos_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_cos"
    }
    list(x = x, y = y, pairwise = pairwise, p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_cosine",
                names = c("cosine", "angular"),
                PREFUN = "pr_cos_prefun",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "xy / sqrt(xx * yy)",
                reference = "Anderberg, M.R. (1973). Cluster Analysis for Applicaitons. Academic Press.",
                description = "The cos Similarity (C implementation)")

pr_eJaccard <- function(x, y) {
    tmp <- crossprod(x, y)
    tmp / (crossprod(x) + crossprod(y) - tmp)
}
pr_eJaccard_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_eJaccard"
    }
    list(x = 0 + x,
         y = if (!is.null(y)) 0 + y else NULL,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_ejaccard",
                names = c("eJaccard", "extended_Jaccard"),
                PREFUN = "pr_eJaccard_prefun",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "xy / (xx + yy - xy)",
                reference = "Strehl A. and Ghosh J. (2000).
Value-based customer grouping from large retail data-sets.
In Proc. SPIE Conference on Data Mining and Knowledge Discovery, Orlando, volume 4057, pages 33-42. SPIE.",
                description = "The extended Jaccard Similarity (C implementation; yields Jaccard for binary x,y).")

pr_eDice <- function(x, y) {
    tmp <- crossprod(x, y)
    tmp / (crossprod(x) + crossprod(y))
}
pr_eDice_prefun <- function(x, y, pairwise, p, reg_entry) {
    if (!is.matrix(x)) {
        reg_entry$C_FUN <- FALSE
        reg_entry$loop <- TRUE
        reg_entry$FUN <- "pr_eDice"
    }
    list(x = 0 + x,
         y = if (!is.null(y)) 0 + y else NULL,
         pairwise = pairwise,
         p = NULL, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "R_edice",
                names = c("eDice", "extended_Dice","eSorensen"),
                PREFUN = "pr_eDice_prefun",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "metric",
                loop = FALSE,
                C_FUN = TRUE,
                abcd = FALSE,
                formula = "2xy / (xx + yy)",
		## FIXME
                reference = "Alexander Strehl. Relationship-based Clustering and Cluster Ensembles for High-dimensional Data Mining. PhD thesis, The University of Texas at Austin, May 2002.",
                description = "The extended Dice Similarity (C implementation; yields Dice for binary x,y).")

pr_cor <- function(x, y) {
    X <- x - mean(x)
    Y <- y - mean(y)
    crossprod(X, Y) / sqrt(crossprod(X) * crossprod(Y))
}
pr_DB$set_entry(FUN = "pr_cor",
                names = "correlation",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "metric",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "xy / sqrt(xx * yy) for centered x,y",
                reference = "Anderberg, M.R. (1973). Cluster Analysis for Applicaitons. Academic Press.",
                description = "correlation (taking n instead of n-1 for the variance)")

pr_ChiSquared <- function(x, y) {
    tab <- table(x,y)
    exp <- rowSums(tab) %o% colSums(tab) / sum(tab)
    sum((tab - exp) ^ 2 / exp)
}
pr_DB$set_entry(FUN = "pr_ChiSquared",
                names = "Chi-squared",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "nominal",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sum_ij (o_i - e_i)^2 / e_i",
                reference = "Anderberg, M.R. (1973). Cluster Analysis for Applicaitons. Academic Press.",
                description = "Sum of standardized squared deviations from observed and expected values in a cross-tab for x and y.")

pr_PhiSquared <- function(x, y) {
    tab <- table(x,y)
    exp <- rowSums(tab) %o% colSums(tab) / sum(tab)
    sum((tab - exp) ^ 2 / exp) / sum(tab)
}
pr_DB$set_entry(FUN = "pr_PhiSquared",
                names = "Phi-squared",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "nominal",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "[sum_ij (o_i - e_i)^2 / e_i] / n",
                reference = "Anderberg, M.R. (1973). Cluster Analysis for Applicaitons. Academic Press.",
                description = "Standardized Chi-Squared (= Chi / n).")

pr_Tschuprow <- function(x, y) {
    tab <- table(x,y)
    exp <- rowSums(tab) %o% colSums(tab) / sum(tab)
    sqrt(sum((tab - exp) ^ 2 / exp) / sum(tab) / sqrt((nrow(tab) - 1) * (ncol(tab) - 1)))
}
pr_DB$set_entry(FUN = "pr_Tschuprow",
                names = "Tschuprow",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "nominal",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt{[sum_ij (o_i - e_i)^2 / e_i] / n / sqrt((p - 1)(q - 1))}",
                reference = "Tschuprow, A.A. (1925). Grundbegriffe und Grundprobleme der Korrelationstheorie. Springer.",
                description = "Tschuprow-standardization of Chi-Squared.")

pr_Cramer <- function(x, y) {
    tab <- table(x,y)
    exp <- rowSums(tab) %o% colSums(tab) / sum(tab)
    sqrt(sum((tab - exp) ^ 2 / exp) / sum(tab) / min((nrow(tab) - 1), (ncol(tab) - 1)))
}
pr_DB$set_entry(FUN = "pr_Cramer",
                names = "Cramer",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "nominal",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt{[Chi / n)] / min[(p - 1), (q - 1)]}",
                reference = "Cramer, H. (1946). The elements of probability theory and some of its applications. Wiley, New York. ",
                description = "Cramer-standization of Chi-Squared.")

pr_Pearson <- function(x, y) {
    tab <- table(x,y)
    exp <- rowSums(tab) %o% colSums(tab) / sum(tab)
    Chi <- sum((tab - exp) ^ 2 / exp)
    sqrt(Chi / (sum(tab) + Chi))
}
pr_DB$set_entry(FUN = "pr_Pearson",
                names = c("Pearson","contingency"),
                distance = FALSE,
                convert = "pr_simil2dist",
                type = "nominal",
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "sqrt{Chi / (n + Chi)}",
                reference = "Anderberg, M.R. (1973). Cluster Analysis for Applicaitons. Academic Press.",
                description = "Contingency Coefficient. Chi is the Chi-Squared statistic.")


pr_Gower <- function(x, y, l = NA, f = NA, m = NA, weights = NA) {
    ## prepare vectors
    len <- length(x)
    d <- logical(len)
    s <- double(len)

    ## compute scores groupwise:

    ## logical
    if (any(l)) {
        s[l] <- x[l] & y[l]
        d[l] <- x[l] | y[l]
    }

    ## factor
    if (any(f)) {
        s[f] <- x[f] == y[f]
        d[f] <- TRUE
    }

    ## metric
    if (any(m)) {
        s[m] <- 1 - abs(as.double(x[m]) - as.double(y[m]))
        d[m] <- TRUE
    }

    ## do not count missings
    d[is.na(s)] <- FALSE
    s[is.na(s)] <- 0

    drop(crossprod(s, weights)) / drop(crossprod(d, weights))
}
pr_Gower_prefun <- function(x, y, pairwise, p, reg_entry) {
    ## transform x and y
    x <- as.data.frame(x)
    if (!is.null(y))
        y <- as.data.frame(y)

    ## determine types
    l <- sapply(x, is.logical)
    f <- sapply(x, is.factor)
    o <- sapply(x, is.ordered)
    f <- f & !o
    m <- !(l | f | o)

    ## Transform ordinal variables
    if (any(o)) {
        ##FIXME: Gower uses ranks, but daisy just uses internal codes??
        x[o] <- lapply(x[o], as.integer)
        x[o] <- lapply(x[o], function(i) (i - 1) / (max(i) - 1))
        if (!is.null(y)) {
            y[o] <- lapply(y[o], as.integer)
            y[o] <- lapply(y[o], function(i) (i - 1) / (max(i) - 1))
        }
        m <- m | o
    }

    ## scale metric types
    RANGE <- function(x) {
        ## compute scale
        ret <- sapply(x,
                      function(i) max(i, na.rm = TRUE) - min(i, na.rm = TRUE))
        ## do not scale when range == 0
        ret[ret == 0] <- 1
        ret
    }
    if (any(m)) {
        if (!is.null(p$ranges) && is.null(p$ranges.x))
            p$ranges.x <- p$ranges
        r <- if(is.null(p$ranges.x))
            RANGE(x[m])
        else
            rep_len(p$ranges.x, length.out = length(m))
        x[m] <- x[m] / rep(r, each = nrow(x))
        if (!is.null(y)) {
            if (!is.null(p$ranges) && is.null(p$ranges.y))
                p$ranges.y <- p$ranges
            r <- if(is.null(p$ranges.y))
                RANGE(y[m])
            else
                rep_len(p$ranges.y, length.out = length(m))
            y[m] <- y[m] / rep(r, each = nrow(y))
        }
    }

    ## weights
    weights <- rep_len(if (is.null(p$weights)) 1 else p$weights,
                   length.out = ncol(x))

    p <- list(l = l, f = f, m = m, weights = weights)
    list(x = x, y = y, pairwise = pairwise, p = p, reg_entry = reg_entry)
}
pr_DB$set_entry(FUN = "pr_Gower",
                names = "Gower",
                PREFUN = "pr_Gower_prefun",
                distance = FALSE,
                convert = "pr_simil2dist",
                type = NA,
                loop = TRUE,
                C_FUN = FALSE,
                abcd = FALSE,
                formula = "Sum_k (s_ijk * w_k) / Sum_k (d_ijk * w_k)",
                reference = "Gower, J.C. (1971). A general coefficient of similarity and some of its properties. Biometrics, 27, pp. 857--871.",
                description = "The Gower Similarity for mixed variable types.
w_k are variable weights. d_ijk is 0 for missings or a pair of FALSE logicals, and 1 else.
s_ijk is 1 for a pair of TRUE logicals or matching factor levels,
and the absolute difference for metric variables.
Each metric variable is scaled with its corresponding range,
provided the latter is not 0.
Ordinal variables are converted to ranks r_i and
the scores z_i = (r_i - 1) / (max r_i - 1) are taken as metric variables.
Note that in the latter case, unlike the definition of Gower, just the
internal integer codes are taken as the ranks, and not what rank() would
return. This is for compatibility with daisy() of the cluster package, and
will make a slight difference in case of ties. The weights w_k
can be specified by passing a numeric vector (recycled as needed) to
the 'weights' argument. Ranges for scaling the columns of x and y
can be specified using the 'ranges.x'/'ranges.y'
arguments (or simply 'ranges' for both x and y).")

###################################
### generic registry infrastructure

### IDEA: use lexical scope with nested functions to create an
### S3-"object" that exposes the data structure only through accessor functions.

.FUNCall <- function(f) function(...) f(...)

registry <-
function(index_field = "names", entry_class = NULL,
         validity_FUN = NULL, registry_class = NULL,
         ignore_case = TRUE)
{
### ATTRIBUTES
    ## repository
    DATA <- META <- list()

    ## permissions
    PERMISSIONS <- c(set_entries = TRUE, modify_entries = TRUE,
                     delete_entries = TRUE, set_fields = TRUE)
    SEALED_FIELDS <- SEALED_ENTRIES <- character(0)


### METHODS (PRIVATE)
    ## helper functions
    .field_exists <-
    function(name)
        name %in% .get_field_names()

    .make_field <-
    function(default = NA, type = NA, is_mandatory = FALSE,
             is_modifiable = TRUE, validity_FUN = NULL)
        structure(list(type = type,
                       default = default,
                       is_mandatory = is_mandatory,
                       is_modifiable = is_modifiable,
                       validity_FUN = validity_FUN),
                  class = "registry_field")

    .make_entry <- function(l)
    {
        ## sort
        l <- l[c(index_field, setdiff(.get_field_names(), index_field))]

        ## return object (possibly inheriting from entry_class)
        structure(l, class = c(entry_class, "registry_entry"))
    }

    .get_mandatory_fields <-
    function()
        names(META)[sapply(META, function(i) i$is_mandatory)]

    .get_field_defaults <-
    function()
        lapply(META, function(i) i$default)

    .get_entry_index <-
    function(name, stop_if_missing = TRUE) {
        ## returns the index of the first exact match (modulo case):
        index <- if (ignore_case)
            sapply(DATA,
                   function(i) toupper(name) %in% toupper(i[[index_field]]))
        else
            sapply(DATA,
                   function(i) name %in% i[[index_field]])
        if (!any(index)) {
            if (stop_if_missing)
                stop(paste("Entry", dQuote(name), "not in registry."))
            else
                return(NULL)
        }
        which(index)[1]
    }

    .check_value <-
    function(field_name, field, value)
    {
        ## Note we do not check NA entries because this may by set automatically
        if (!is.function(value) && !any(is.na(value))) {
            ## check class / list of alternatives, if any
            if (!any(is.na(field$type))) {
                ## check list of alternatives
                if (length(field$type) > 1) {
                    if (!is.character(value) || !value %in% field$type)
                        stop(paste("Possible values for", dQuote(field_name), "are:",
                                   paste(field$type, collapse = ", ")))
                ## check class
                } else if (!inherits(value, field$type)) {
                    stop(paste("Field", dQuote(field_name), "does not inherit from class", field$type))
                }
            }

            ## apply validity function, if any
            if (!is.null(field$validity_FUN))
                do.call(field$validity_FUN, list(value))
        }
    }

    .check_for_unknown_fields <-
    function(n)
    {
        ## check for fields not in repository
        missing <- !.field_exists(n)
        if (any(missing))
            stop(paste("Field(s) not in repository:", paste(n[missing], collapse = ", ")))
    }

### METHODS (PUBLIC)
    ## field accessors
    .entry_exists <-
    function(name)
        if (ignore_case)
            toupper(name) %in% toupper(unlist(sapply(DATA, function(i) i[[index_field]])))
        else
            name %in% unlist(sapply(DATA, function(i) i[[index_field]]))

    .get_field <-
    function(name)
    {
        if (!.field_exists(name))
            stop(paste("Field", dQuote(name), "not in registry."))

        META[[name]]
    }

    .get_fields <-
    function()
        META

    .get_field_names <-
    function()
        names(META)

    .set_field <-
    function(name, default = NA, type = NA, is_mandatory = FALSE,
             is_modifiable = TRUE, validity_FUN = NULL)
    {
        ## check permissions
        if (!PERMISSIONS["set_fields"])
            stop("Setting of fields not allowed.")

        ## check for double entries
        if (.field_exists(name))
            stop(paste("Field", dQuote(name), "already in registry."))

        ## possibly, infer type from argment
        if (!any(is.na(type)) && !(is.character(type)))
            type <- class(type)

        ## check mandatory fields
        if (is_mandatory && !any(is.na(default)))
            stop("Mandatory fields should have no default.")

        ## create field entry
        field <- .make_field(type = type,
                             default = default,
                             is_mandatory = is_mandatory,
                             validity_FUN = validity_FUN)

        ## check validity of default
        .check_value("default", field, default)

        ## add field to meta data
        META <<- c(META, list(field))
        names(META)[length(META)] <<- name

        ## add (missing) fields to data entries
        DATA <<- lapply(DATA, function(i) {i[[name]] <- default; i})
    }

    .n_of_entries <-
    function()
        length(DATA)

    ## entry accessors
    .set_entry <-
    function(...)
    {
        ## check permissions
        if (!PERMISSIONS["set_entries"])
            stop("Setting of entries not allowed.")

        ## parameter handling
        l <- list(...)
        n <- names(l)

        .check_for_unknown_fields(n)

        ## check for mandatory fields
        mandatory_fields <- .get_mandatory_fields()
        missing_mandatory_fields <- !mandatory_fields %in% n
        if (any(missing_mandatory_fields))
            stop(paste("The following fields are mandatory, but missing:",
                       paste(mandatory_fields[missing_mandatory_fields], collapse = ", ")))

        ## check for double entries
        for (i in l[[index_field]])
            if (.entry_exists(i))
                stop(paste("Entry", dQuote(i), "already in registry."))

        ## check defaults and set values, if needed
        field_defaults    <- .get_field_defaults()
        default_fields    <- names(field_defaults)
        missing_fields    <- setdiff(default_fields, n)
        l[missing_fields] <- field_defaults[missing_fields]

        ## check field types, and apply field check function, if any.
        for (f in n) {
            meta <- .get_field(f)
            .check_value(f, .get_field(f), l[[f]])
        }

        ## apply entry check function
        if (!is.null(validity_FUN))
            do.call(validity_FUN, list(l))

        ## add entry
        entry <- .make_entry(l)
        DATA <<- c(DATA, list(entry))
        names(DATA)[length(DATA)] <<- l[[index_field]][1]
    }

    .get_entries <-
    function(names = NULL, pattern = NULL) {
        ## fix search
        if (!is.null(names)) {
            if (ignore_case)
                DATA[intersect(toupper(names), toupper(names(DATA)))]
            else
                DATA[intersect(names, names(DATA))]
        ## grep search
        } else if (!is.null(pattern)) {
            pattern_in_entry <-
                function(x) any(sapply(x, function(i) is.character(i)
                                       && length(grep(pattern, i) > 0)))
            DATA[sapply(DATA, pattern_in_entry)]
        ## else: return all entries
        } else DATA

    }

    .get_entry_names <-
    function()
    {
        if (length(DATA) < 1)
            character(0)
        else
            names(DATA)
    }

    .get_entry <-
    function(name, stop_if_missing = TRUE)
    {
        index <- .get_entry_index(name, stop_if_missing)
        if (is.null(index))
            return(NULL)
        DATA[[index]]
    }

    .delete_entry <-
    function(name)
    {
        ## check permissions
        if (!PERMISSIONS["delete_entries"])
            stop("Deletion of entries not allowed.")

        ## fetch entry index (this also checks if the entry exists)
        entry_index <- .get_entry_index(name)

        ## check sealed entries
        if (name %in% SEALED_ENTRIES)
            stop(paste("Deletion of entry", dQuote(name), "not allowed."))

        ## delete it
        DATA[entry_index] <<- NULL
    }


    .modify_entry <-
    function(...)
    {
        ## check permissions
        if (!PERMISSIONS["modify_entries"])
            stop("Modifying of entries not allowed.")

        ## parameter handling
        l <- list(...)
        n <- names(l)

        ## check for index field
        if (!index_field %in% n)
            stop(paste("Index field", dQuote(index_field), "missing."))

        .check_for_unknown_fields(n)

        ## determine entry name
        name <- l[[index_field]][1]

        ## fetch entry index (this also checks if the entry exists)
        entry_index <- .get_entry_index(name)

        ## fetch entry
        entry <- DATA[[entry_index]]
        name <- entry[[index_field]][1]

        for (field in setdiff(n, index_field)) {
            ## check if field is modifiable
            field_entry <- .get_field(field)
            if (!field_entry$is_modifiable)
                stop(paste("Field", dQuote(field), "is not modifiable."))

            ## check if entry and field are sealed
            if ((name %in% SEALED_ENTRIES) && (field %in% SEALED_FIELDS))
                stop(paste("Modification of field", dQuote(field),
                           "in entry", dQuote(name), "not allowed."))

            ## check new value
            value <- l[[field]]
            .check_value(field, field_entry, value)

            ## modify entry locally
            entry[[field]] <- value
        }

        ## apply entry check function
        if (!is.null(validity_FUN))
            do.call(validity_FUN, list(entry))

        ## modify entry in registry
        DATA[entry_index] <<- list(entry)
    }

    ## get all entries for one field
    .get_field_entries <-
    function(field, unlist = TRUE)
    {
        if (!.field_exists(field))
            stop(paste("Field", dQuote(field), "not in registry."))

        ret <- lapply(DATA, function(i) i[[field]])
        if (unlist)
            unlist(ret)
        else
            ret
    }

    ## permission getters/setters
    .get_permissions <-
    function()
        PERMISSIONS

    .restrict_permissions <-
    function(set_entries = TRUE, modify_entries = TRUE,
             delete_entries = TRUE, set_fields = TRUE)
    {
        PERMISSIONS["set_entries"] <<- PERMISSIONS["set_entries"] && set_entries
        PERMISSIONS["modify_entries"] <<- PERMISSIONS["modify_entries"] && modify_entries
        PERMISSIONS["delete_entries"] <<- PERMISSIONS["delete_entries"] && delete_entries
        PERMISSIONS["set_fields"] <<- PERMISSIONS["set_fields"] && set_fields
    }

    .seal_entries <-
    function()
    {
        SEALED_ENTRIES <<- .get_entry_names()
        SEALED_FIELDS <<- .get_field_names()
    }

    .get_sealed_entry_names<-
    function()
        SEALED_ENTRIES

    .get_sealed_field_names <-
    function()
        SEALED_FIELDS

### CONSTRUCTOR

    ## create index field
    .set_field(name = index_field, type = "character",
               is_mandatory = TRUE, is_modifiable = FALSE)

    ## return class
    structure(list(get_field = .FUNCall(.get_field),
                   get_fields = .FUNCall(.get_fields),
                   get_field_names = .FUNCall(.get_field_names),
                   set_field = .FUNCall(.set_field),

                   entry_exists = .FUNCall(.entry_exists),
                   get_entry = .FUNCall(.get_entry),
                   get_entries = .FUNCall(.get_entries),
                   get_entry_names = .FUNCall(.get_entry_names),
                   set_entry = .FUNCall(.set_entry),
                   modify_entry = .FUNCall(.modify_entry),
                   delete_entry = .FUNCall(.delete_entry),
                   n_of_entries = .FUNCall(.n_of_entries),

                   get_field_entries = .FUNCall(.get_field_entries),

                   get_permissions = .FUNCall(.get_permissions),
                   restrict_permissions = .FUNCall(.restrict_permissions),
                   seal_entries = .FUNCall(.seal_entries),
                   get_sealed_entry_names = .FUNCall(.get_sealed_entry_names),
                   get_sealed_field_names = .FUNCall(.get_sealed_field_names)
                   ),
              class = c(registry_class, "proxy_registry"))
}

"[[.proxy_registry" <-
function(x, i)
  x$get_entry(i)

length.proxy_registry <-
function(x)
    x$n_of_entries()

print.proxy_registry <-
function(x, ...)
{
    l <- x$n_of_entries()
    if (l < 1)
        writeLines(paste("An object of class", dQuote("registry"), "with no entry."))
    else if (l == 1)
        writeLines(paste("An object of class", dQuote("registry"), "with one entry."))
    else
        writeLines(paste("An object of class", dQuote("registry"), "with", l, "entries."))
}

print.registry_field <-
function(x, ...)
    writeLines(formatUL(x, label = names(x), ...))

print.registry_entry <-
function(x, ...)
{
    x <- .functions_to_characters(x)
    x[[1]] <- paste(x[[1]], collapse = ", ")
    writeLines(formatUL(x, label = names(x)))
}

summary.proxy_registry <-
function(object, ...)
    as.data.frame(object, ...)

as.data.frame.proxy_registry <-
function(x, ...)
    do.call(rbind,
            lapply(x$get_entries(),
                   function(entry) {
                       entry <- .functions_to_characters(entry)
                       data.frame(unclass(entry[-1]), ...)
                   }
                   )
            )

.functions_to_characters <-
function(x)
{
    ## transform function entries into character strings
    funs <- sapply(x, inherits, "function")
    for (field in names(x)[funs])
        x[[field]] <- paste(format(x[[field]]), collapse = "")
    x
}

\documentclass{article}
\title{Proximity measures in the \texttt{proxy} package for {\sf R}}
\author{David Meyer}

%\VignetteIndexEntry{Proximity measures in the proxy package for R}
%\VignetteDepends{proxy}
%\VignetteKeywords{similarities,dissimilarities,distance}
%\VignettePackage{proxy}

<<preliminaries,echo=FALSE,results=hide>>=
library(proxy)
x <- summary(pr_DB, "long")
FUN <- function(index) {
    for (i in which(index)) {
        writeLines(sprintf("Aliases: %s", paste(x$names[[i]], collapse = ", ")))
        writeLines(sprintf("Type   : %s", x$type[i]))
        writeLines(sprintf("Formula: %s\n", x$formula[i]))
    }
}
@
\begin{document}
\maketitle
\section{Similarities}
<<echo=FALSE>>=
FUN(x$distance == FALSE)
@

\section{Dissimilarities}
<<echo=FALSE>>=
FUN(x$distance == TRUE)
@
\end{document}

\name{rowSums.dist}
\alias{rowSums.dist}
\alias{colSums.dist}
\alias{rowMeans.dist}
\alias{colMeans.dist}
\title{Row Sums/Means of Sparse Symmetric Matrices}
\description{
  Compute the row (column) sums or means for a sparse symmetric 
  (distance) matrix.
}
\usage{
rowSums.dist(x, na.rm = FALSE)
rowMeans.dist(x, na.rm = FALSE, diag = TRUE)

colSums.dist(x, na.rm = FALSE)
colMeans.dist(x, na.rm = FALSE, diag = TRUE)
}
\arguments{
  \item{x}{an object of class \code{dist}.}
  \item{na.rm}{logical, should missing values (including \code{NaN}) be
    omitted from the summation?}
  \item{diag}{logical, should the diagonal elements be included
    in the computation?}
}
\details{
  These functions are more efficient than expanding an object of
  class \code{dist} to matrix and using \code{rowSums} or \code{rowMeans}.

  \code{colSums} and \code{colMeans} are provided for convenience.
  However, note that due to symmetry the result is always the
  same as for \code{rowSums} or \code{rowMeans}.
}
\value{
  A numeric vector of row sums.
}
\author{Christian Buchta}
\seealso{\code{as.matrix}, \code{as.dist}, and \code{rowSums}.}
\examples{
## 
x <- matrix(runif(10*2),ncol=2)
d <- dist(x)
rowSums(as.matrix(d))
rowSums.dist(d)			# the same

rowMeans(as.matrix(d))
rowMeans.dist(d)		# the same
rowMeans.dist(d, diag = FALSE)	# not the same
## NAs
d[3] <- NA
rowSums.dist(d, na.rm = TRUE)
rowMeans.dist(d, na.rm = TRUE)
}
\keyword{cluster}

\name{pr_DB}
\alias{pr_DB}
\alias{registry}
\alias{summary.pr_DB}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Registry of proximities}
\description{
  Registry containing similarities and distances.
}
\usage{
pr_DB
\special{pr_DB$get_field(name)}
\special{pr_DB$get_fields()}
\special{pr_DB$get_field_names()}
\special{pr_DB$set_field(name, default = NA, type = NA, is_mandatory = FALSE,
                is_modifiable = TRUE, validity_FUN = NULL)}

\special{pr_DB$entry_exists(name)}
\special{pr_DB$get_entry(name)}
\special{pr_DB$get_entries(name = NULL, pattern = NULL)}
\special{pr_DB$get_entry_names(name)}
\special{pr_DB$set_entry(...)}
\special{pr_DB$modify_entry(...)}
\special{pr_DB$delete_entry(name)}

\method{summary}{pr_DB}(object, verbosity = c("short", "long"), ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{name}{character string representing the name of an
    entry (case-insensitive).}
  \item{pattern}{regular expression to be matched to all fields of class
    \code{"character"} in all entries.}
  \item{default}{optional default value for the field.}
  \item{type}{optional character string specifying the class to be
    required for this field. If \code{type} is a character vector with more
    than two elements, the entries will be used as fixed set of
    alternatives. If \code{type} is not a character string or vector, the
    class will be inferred from the argument given.}
  \item{is_mandatory}{logical specifying whether new entries are required
    to have a value for this field.}
  \item{is_modifiable}{logical specifying whether entries can be changed
    with respect to that field.}
  \item{validity_FUN}{optional function or character string with the name of a
    function that checks the validity of a field entry. Such a function
    gets the value to be investigated as argument, and should stop with an
    error message if the value is not correct.}
  \item{object}{a registry object.}
  \item{verbosity}{controlling the verbosity of the output of the
    summary method for the registry. \code{"short"} gives just a list, \code{"long"}
    also gives the formulas.}
  \item{\dots}{for \code{pr_DB$set_entry} and \code{pr_DB$modify_entry}:
    named list of fields to be modified in or added to the registry (see details).
    This must include the index field (\code{"names"}).}
}
\details{
  \code{pr_DB} represents the registry of all proximity measures
  available. For each
  measure, it comprises meta-information that can be queried and
  extended. Also, new measures can be added. This is done using
  the following accessor functions of the \code{pr_DB} object:
  
  \code{get_field_names()} returns a character
  vector with all field names. \code{get_field()} returns the information
  for a specific field as a list with components named as described
  above. \code{get_fields()} returns a list with all field
  entries. \code{set_field()} is used to create new fields in the
  repository (the default value will be set in all
  entries).

  \code{get_entry_names()} returns a character vector with (the first
  alias of) all entries. \code{entry_exists()} is a predicate checking
  if an entry with the specified alias exists in the
  registry. \code{get_entry()} returns the specified entry if it exists (and, by
  default, gives an error if it does not). \code{get_entries()} is used to
  query more than one entry: either those matching \code{name} exactly, or
  those where the regular expression in \code{pattern} matches \emph{any}
  character field in an entry. By default, all values are
  returned. \code{delete_entry} removes an existing entry from the
  registry (note that only user-provided entries can be deleted).
  \code{set_entry} and \code{modify_entry} require a named list
  of arguments used as field entries.
  At least the \code{names} index field is required. \code{set_entry}
  will check for all other mandatory fields. If specified in the field
  meta data, each field entry and the entry as a whole is checked for
  validity. Note that only user-specified fields and/or entries can be
  modified, the data shipped with the package are read-only.

  The registry fields currently available are as follows:
  
\describe{
  \item{FUN}{Function to register (see below).}
  \item{names}{Character vector with an alias(es) for the measure.}
  \item{PREFUN}{Optional function (or function name) for preprocessing
    code (see below).}
  \item{POSTFUN}{Optional function (or function name) for postprocessing
    code (see below).}
  \item{distance}{logical indicating whether this measure is a distance (\code{TRUE})
    or similarity (\code{FALSE}).}
  \item{convert}{Optional Function or function name for converting
    between similarities and distances when needed.}
  \item{type}{Optional, the scale the measure applies to
    (\code{"metric"}, \code{"ordinal"}, \code{"nominal"},
  \code{"binary"}, or \code{"other"}). If
    \code{NULL}, it is assumed to apply to some other unknown scale.}
  \item{loop}{logical indicating whether \code{FUN} is just a measure,
    and therefore, if \code{\link{dist}} shall do the loop over all pairs of
    observations/variables, or if \code{FUN} does the loop on its own.}
  \item{\code{C_FUN}}{logical indicating whether \code{FUN} is a C function.}
  \item{abcd}{logical; if \code{TRUE} and binary data (or data to be
    interpreted as such) are supplied, the number of concordant and
    discordant pairs is precomputed for every two binary data vectors and
    supplied to the measure function.}
  \item{formula}{Optional character string with the symbolic representation of
    the formula.}
  \item{reference}{Optional reference (character).}
  \item{description}{Optional description (character). Ideally,
    describes the context in which the measure can be applied.}  
}

A function specified as \code{FUN} parameter has mandatory arguments
\code{x} and \code{y} (if \code{abcd} is \code{FALSE}), and \code{a},
\code{b}, \code{c}, \code{d}, \code{n} otherwise. Additionally, it gets
all optional parameters specified by the user in the \code{\dots}
argument of the \code{\link{dist}} and \code{\link{simil}} functions, possibly
changed and/or complemented by the corresponding (optional)
\code{PREFUN} function. It must return the
(diss-)similarity value computed from the arguments.
\code{x} and \code{y} are two vectors from the
data matrix (matrices) supplied. If \code{abcd} is \code{FALSE}, it is
assumed that binary measures will be used, and the number of all
\code{n} concordant and discordant pairs (x\_k, y\_k)
precomputed and supplied instead of \code{x} and
\code{y}. \code{a}, \code{b}, \code{c}, and \code{d} are the counts of
all (TRUE, TRUE), (TRUE, FALSE), (FALSE, TRUE), and (FALSE, FALSE)
pairs, respectively.

A function specified as \code{PREFUN} parameter has mandatory arguments
\code{x}, \code{y}, \code{p}, and \code{reg_entry}, with \code{y} and
\code{p} possibly being \code{NULL} depending on the task at
hand. \code{x} and \code{y} are the data objects, \code{p} is a
(possibly empty) list with all specified proximity parameters, and
\code{reg_entry} is the registry entry (a named list containing all
information specified in \code{reg_add}).
The preprocessing function is allowed to change all these
information, and if so, is required to return *all* arguments
as a named list in the same order.

A function specified as \code{POSTFUN} parameter has two mandatory
arguments: \code{result} and \code{p}. \code{result} will contain the
computed raw data, i.e. a vector of length \eqn{n * (n - 1) / 2} for
auto-distances (see \code{\link[stats]{dist}} for details on
\code{dist} objects), or a matrix for cross-distances. \code{p} contains
the specified proximity parameters. Post-processing functions need to
return the \code{result} object (even if unmodified).

A function specified as \code{convert} parameter should preserve the
type of its argument.
}
\author{David Meyer \email{David.Meyer@R-project.org}}

\seealso{\code{\link{dist}}}
\examples{
## create a new distance measure
mydist <- function(x,y) x * y

## create a new entry in the registry with two aliases
pr_DB$set_entry(FUN = mydist, names = c("test", "mydist"))

## look it up (index is case insensitive):
pr_DB$get_entry("TEST")

## modify the content of the description field in the new entry
pr_DB$modify_entry(names = "test", description = "foo function")

## create a new field
pr_DB$set_field("New")

## look up the test entry again (two ways)
pr_DB$get_entry("test")
pr_DB[["test"]]

## show total number of entries
length(pr_DB)

## show all entries (short list)
pr_DB$get_entries(pattern = "foo")

## show more details
summary(pr_DB, "long")

## get all entries in a list (and extract first two ones)
pr_DB$get_entries()[1:2]

## get all entries as a data frame (select first 3 fields)
as.data.frame(pr_DB)[,1:3]

## delete test entry
pr_DB$delete_entry("test")

## check if it is really gone
pr_DB$entry_exists("test")

}
\keyword{cluster}% __ONLY ONE__ keyword per line

\name{dist}
\alias{dist}
\alias{simil}
\alias{print.simil}
\alias{print.dist}
\alias{print.crosssimil}
\alias{print.crossdist}
\alias{as.matrix.dist}
\alias{as.matrix.simil}
\alias{pr_simil2dist}
\alias{pr_dist2simil}
\alias{as.matrix}
\alias{as.dist}
\alias{as.simil}
\alias{row.dist}
\alias{col.dist}
\title{Matrix Distance/Similarity Computation}
\description{
  These functions compute and return the auto-distance/similarity matrix
  between either rows or columns of a matrix/data frame, or a list,
  as well as the cross-distance matrix between two matrices/data frames/lists.
}
\usage{
dist(x, y = NULL, method = NULL, ..., diag = FALSE, upper = FALSE,
     pairwise = FALSE, by_rows = TRUE, convert_similarities = TRUE,
     auto_convert_data_frames = TRUE)
simil(x, y = NULL, method = NULL, ..., diag = FALSE, upper = FALSE,
      pairwise = FALSE, by_rows = TRUE, convert_distances = TRUE,
      auto_convert_data_frames = TRUE)

pr_dist2simil(x)
pr_simil2dist(x)

as.dist(x, FUN = NULL)
as.simil(x, FUN = NULL)

\method{as.matrix}{dist}(x, diag = 0, \dots)
\method{as.matrix}{simil}(x, diag = NA, \dots)
}
\arguments{
  \item{x}{For \code{dist} and \code{simil}, a numeric matrix object, a data frame, or a list. A vector
    will be converted into a column matrix. For \code{as.simil} and
    \code{as.dist}, an object of class \code{dist} and
    \code{simil}, respectively, or a numeric matrix. For
    \code{pr_dist2simil} and \code{pr_simil2dist}, any numeric vector.}
  \item{y}{\code{NULL}, or a similar object than \code{x}}
  \item{method}{a function, a registry entry, or a mnemonic string referencing the
    proximity measure. A list of all available measures can be obtained
    using \code{\link{pr_DB}} (see examples). The default for \code{dist} is
    \code{"Euclidean"}, and for \code{simil} \code{"correlation"}.}
  \item{diag}{logical value indicating whether the diagonal of the
    distance/similarity matrix should be printed by
    \code{\link{print.dist}}/\code{\link{print.simil}}. Note that the
    diagonal values are never stored in \code{dist} objects. 

    In the context of \code{as.matrix} the value to use on the diagonal
    representing self-proximities. In case of similarities, this
    defaults to \code{NA} since a priori there are no upper bounds, so
    the maximum similarity needs to be specified by the user.}
  \item{upper}{logical value indicating whether the upper triangle of the
    distance/similarity matrix should be printed by
    \code{\link{print.dist}}/\code{\link{print.simil}}}
  \item{pairwise}{logical value indicating whether distances should be
    computed for the pairs of \code{x} and \code{y} only.}
  \item{by_rows}{logical indicating whether proximities between rows, or
    columns should be computed.}
  \item{convert_similarities, convert_distances}{logical indicating
    whether distances should be automatically converted into
    similarities (and the other way round) if needed.}
  \item{auto_convert_data_frames}{logical indicating whether data frames
    should be converted to matrices if all variables are numeric,
    or all are logical, or all are complex.}
  \item{FUN}{optional function to be used by \code{as.dist} and
    \code{as.simil}. If \code{NULL}, it is looked up in the method
    registry. If there is none specified there, \code{FUN} defaults to
    \code{pr_simil2dist} and \code{pr_dist2simil}, respectively.}
  \item{\dots}{further arguments passed to the proximity function.}
}
\details{
  The interface is fashioned after \code{\link[stats]{dist}}, but can
  also compute cross-distances, and allows user extensions by means of
  registry of all proximity measures (see \code{\link{pr_DB}}).

  Missing values are allowed but are excluded from all computations 
  involving the rows within which they occur. If some columns are
  excluded in calculating a Euclidean, Manhattan, Canberra or
  Minkowski distance, the sum is scaled up proportionally to the
  number of columns used (compare \code{\link[stats]{dist}} in
  package \pkg{stats}).

  Data frames are silently coerced to matrix if all columns are of
  (same) mode \code{numeric} or \code{logical}.

  Distance measures can be used with \code{simil}, and similarity
  measures with \code{dist}. In these cases, the result is transformed
  accordingly using the specified coercion functions (default:
  \eqn{pr\_simil2dist(x) = 1 - abs(x)} and \eqn{pr\_dist2simil(x) = 1 / (1 + x)}).
  Objects of class \code{simil} and \code{dist} can be converted one in
  another using \code{as.dist} and \code{as.simil}, respectively.

  Distance and similarity objects can conveniently be subset
  (see examples). Note that duplicate indexes are silently ignored.
}
\value{
  Auto distances/similarities are returned as an object of class \code{dist}/\code{simil} and 
  cross-distances/similarities as an object of class \code{crossdist}/\code{crosssimil}. 
}
\references{
  Anderberg, M.R. (1973), \emph{Cluster analysis for applications},
  359 pp., Academic Press, New York, NY, USA.
  
  Cox, M.F. and Cox, M.A.A. (2001), \emph{Multidimensional Scaling},
  Chapman and Hall.
  
  Sokol, R.S. and Sneath P.H.A (1963), \emph{Principles of Numerical
  Taxonomy}, W. H. Freeman and Co., San Francisco.
}
\author{David Meyer \email{David.Meyer@R-project.org}
  and Christian Buchta \email{Christian.Buchta@wu-wien.ac.at}}

\seealso{\code{\link[stats]{dist}} for compatibility information, and
  \code{\link{pr_DB}} for the proximity data base.}
\examples{
### show available proximities
summary(pr_DB)

### get more information about a particular one
pr_DB$get_entry("Jaccard")

### binary data
x <- matrix(sample(c(FALSE, TRUE), 8, rep = TRUE), ncol = 2)
dist(x, method = "Jaccard")

### for real-valued data
dist(x, method = "eJaccard")

### for positive real-valued data
dist(x, method = "fJaccard")

### cross distances
dist(x, x, method = "Jaccard")

### pairwise (diagonal)
dist(x, x, method = "Jaccard", 
	 pairwise = TRUE)

### this is the same but less efficient
as.matrix(stats::dist(x, method = "binary"))

### numeric data
x <- matrix(rnorm(16), ncol = 4)

## test inheritance of names
rownames(x) <- LETTERS[1:4]
colnames(x) <- letters[1:4]
dist(x)
dist(x, x)

## custom distance function
f <- function(x, y) sum(x * y)
dist(x, f)

## working with lists
z <- unlist(apply(x, 1, list), recursive = FALSE)
(d <- dist(z))
dist(z, z)

## subsetting
d[[1:2]]
subset(d, c(1,3,4))
d[[c(1,2,2)]]	    # duplicate index gets ignored

## transformations and self-proximities
as.matrix(as.simil(d, function(x) exp(-x)), diag = 1)

## row and column indexes
row.dist(d)
col.dist(d)
}
\keyword{cluster}
